C----------------------------------------------------------------
      PROGRAM wf2f2000
      implicit none
C----------------------------------------------------------------

C   program to read output files in WF format (produced e.g. by
C   WimpSimp or WimpAnn)
c   Author: Joakim Edsjo, edsjo@physto.se
c   Date: 2001-04-03

      include 'wf.inc'
      include 'w2f.inc'
      include 'w2f.ini'
      include 'rdmc.inc'

      integer i,j,result,ok,good,ngood,ngoodmax
      real*8 th,phi,murang,a,b,v_rad,v_z,v_phi,distuni,
     &  nu_th,nu_phi,nu_th0,nu_th1,nu_th2,
     &  numu_th,numu_phi,mu_th,mu_phi,nu_t,
     &  x0,y0,z0,nx,ny,nz,lmu,lmin,rmin,zmax
      integer ranseed
      common /w2f_ran/ranseed
      save /w2f_ran/


c----------------------------------------------------------------- Setup

c      th1=30.0d0
c      phi1=230.0d0
c      th2=120.0d0
c      do i=0,16
c        phi2=dble(i)/16.0d0*360.0d0
c        call angadd(th1,phi1,th2,phi2,th3,phi3)
c        write(*,'(6(1x,F8.3))') th1,phi1,th2,phi2,th3,phi3
c      enddo
c      stop

c...Startup

      call startup

c...Check that detector geometry is OK
      call checkgeom
      
c...Open WF file for input
      write(*,*) ' '
      write(*,*) 'Input file name (in WF 1.0 format):'
      read(*,'(a)') wffile
      write(*,*) wffile
      
c...Determine optimal volume parameters
      call simvol(a,b)

      open(unit=luwf,file=wffile,status='old',form='formatted')
      read(luwf,'(1x,a)') wfver
      read(luwf,'(1x,a)') wfprog
      read(luwf,'(1x,a)') wfpara
      read(luwf,'(1x,a)') wfdate

      if (wfver.ne.'WF 1.0') then
        write(*,*)
     &        'WARNING: A different version of the WF format found.'
        write(*,*) 'Please check the code for consistency.'
      endif

      read(wfpara,5010) wfmneu,wfwh,wfch,wfnmuon

      if (wfwh.ne.'ea'.and.wfwh.ne.'EA') then
        write(*,*) ' '
        write(*,*)
     &   'The file you have specified is for annihilation in the Sun.' 
        write(*,*) 
     &   'In what way do you want to distribute the neutrinos?'
        write(*,*) '  1 = with a fix theta'
        write(*,*) '  2 = with a distribution of theta according to'
        write(*,*) '      the time the Sun spends at different angles.'
        read(*,*) nu_way
        write(*,*) nu_way
        
        if (nu_way.eq.1) then
 75      write(*,*) 'Enter zenith angle theta:'
          read(*,*) nu_th0
          write(*,*) nu_th0
          if ((nu_th0.gt.(90.0d0+w2f_suinc)).or.
     &      (nu_th0.lt.(90.0d0-w2f_suinc))) then
            write(*,*) 'You have entered an unphysical angle.'
            write(*,'(A,F6.2,A,F6.2,A)') 'The Sun is always between ',
     &        90.0d0-w2f_suinc,' and ',90.0d0+w2f_suinc,
     &        ' degrees.'
            write(*,'(A,F6.2,A)') 'Do you want to keep your choice ',
     &        nu_th0,' anyway? (1=yes, 0=no)'
            read(*,*) ok
            write(*,*) ok
            if (ok.ne.1) goto 75
          endif
        elseif (nu_way.eq.2) then
          write(*,*) 'A start and an end time is needed:'
          write(*,*) 
     & '  time = 0.00 - the Sun is max above the horizon.'
          write(*,*)
     & '  time = 0.25 - the Sun is at the horizon'
          write(*,*)
     & '  time = 0.50 - the Sun is max below the horizon.'
          write(*,*)
     & '  time = 0.75 - the Sun is at the horizon'
          write(*,*)
     & '  time = 1.00 - the Sun is max above the horizon.'
          write(*,*) 'Enter start time, t1, and end time, t2:'
          read(*,*) nu_t1,nu_t2
          write(*,*) nu_t1,nu_t2

          nu_th1=90.0d0-w2f_suinc*cos(nu_t1*2*pi)
          nu_th2=90.0d0-w2f_suinc*cos(nu_t2*2*pi)
          write(*,*) 'Zenith angle for t1: ',nu_th1
          write(*,*) 'Zenith angle for t2: ',nu_th2
        endif

      endif

      write(*,*) ' '
      write(*,*) 'Neutralino mass: ',wfmneu 
      write(*,*) 'Annihilation place: ',wfwh
      write(*,*) 'Annihilation channel: ',wfch
      write(*,*) 'Number of muons: ',wfnmuon

C---------------------------------------------------- SIMULATION VOLUMES
      if (wfwh.eq.'ea'.or.wfwh.eq.'EA') then   ! Earth

c...Calculate safe volume
        w2f_maxrang=murang(wfmneu)
        w2f_szmax=w2f_ztop+10
        w2f_szmin=w2f_zbot-w2f_maxrang*1.1d0-25.0d0
        w2f_srad=(w2f_maxrang*sin(w2f_thmax*pi/180.0d0)*1.1d0
     &    +w2f_detrad)+25.0d0

c...Calculate optimal volume
        w2f_ozmax=w2f_ztop+10.0
        w2f_ozmin=w2f_zbot-b*1.1d0-25.0
        w2f_orad=w2f_detrad+a*1.1d0+25.0
      
c...Round-off
        w2f_szmax=int(w2f_szmax+0.5d0)
        w2f_szmin=int(w2f_szmin-0.5d0)
        w2f_srad =int(w2f_srad+0.5d0)
        w2f_ozmax=int(w2f_ozmax+0.5d0)
        w2f_ozmin=int(w2f_ozmin-0.5d0)
        w2f_orad =int(w2f_orad+0.5d0)

c...Suggested volume
        w2f_zmax=int((w2f_ozmax+5.0d0)/10.0d0)*10.0d0
        w2f_zmin=int((w2f_ozmin-10.0d0)/10.0d0)*10.0d0
        w2f_rad =max(int((w2f_orad+10.0d0)/10.0d0)*10.0d0,
     &    w2f_detrad+70.0d0)

      else                                     ! Sun

c...Calculate safe volume
        w2f_maxrang=murang(wfmneu)
        w2f_szmax=w2f_ztop+w2f_maxrang
     &    *sin((abs(w2f_thmax)+abs(w2f_suinc))*pi/180.0d0)*1.1d0
     &    +25.0d0 
        w2f_szmin=w2f_zbot-w2f_maxrang
     &    *sin((abs(w2f_thmax)+abs(w2f_suinc))*pi/180.0d0)*1.1d0
     &    -25.0d0 
        w2f_srad=w2f_detrad+w2f_maxrang*1.1d0+25.0d0

c...Calculate optimal volume
        w2f_ozmax=w2f_ztop+b*1.1d0+25.0
        w2f_ozmin=w2f_zbot-b*1.1d0-25.0
        w2f_orad=w2f_detrad+a*1.1d0+25.0
      
c...Round-off
        w2f_szmax=int(w2f_szmax+0.5d0)
        w2f_szmin=int(w2f_szmin-0.5d0)
        w2f_srad =int(w2f_srad+0.5d0)
        w2f_ozmax=int(w2f_ozmax+0.5d0)
        w2f_ozmin=int(w2f_ozmin-0.5d0)
        w2f_orad =int(w2f_orad+0.5d0)

c...Suggested volume
        w2f_zmax=int((w2f_ozmax+5.0d0)/10.0d0)*10.0d0
        w2f_zmin=int((w2f_ozmin-10.0d0)/10.0d0)*10.0d0
        w2f_rad =max(int((w2f_orad+10.0d0)/10.0d0)*10.0d0,
     &    w2f_detrad+70.0d0)

      endif
      
      write(*,*) ' '
      write(*,*) 'Maximum muon range: ',w2f_maxrang
      write(*,*) ' '
      write(*,*) '### 100% safe volume for all files with this mass ###'
      write(*,*) '   z_max  = ',w2f_szmax
      write(*,*) '   z_min  = ',w2f_szmin
      write(*,*) '   radius = ',w2f_srad
      write(*,'(A,F9.6)') '    Detector volume / generation volume = ',
     &  ((w2f_ztop-w2f_zbot)*pi*w2f_detrad**2) / 
     &  ((w2f_szmax-w2f_szmin)*pi*w2f_srad**2)

      write(*,*) ' '
      write(*,*) '### Optimal (100% safe) for this data file ###'
      write(*,*) '   z_max  = ',w2f_ozmax
      write(*,*) '   z_min  = ',w2f_ozmin
      write(*,*) '   radius = ',w2f_orad
      write(*,'(A,F9.6)') '    Detector volume / generation volume = ',
     &  ((w2f_ztop-w2f_zbot)*pi*w2f_detrad**2) / 
     &  ((w2f_ozmax-w2f_ozmin)*pi*w2f_orad**2)
      
            
      write(*,*) ' '
      write(*,*) '### Suggested volume to spread vertices in ###'
 5    write(*,*) '   z_max = ',w2f_zmax
      write(*,*) '   z_min = ',w2f_zmin
      write(*,*) '   radius = ',w2f_rad
      write(*,'(A,F9.6)') '    Detector volume / generation volume = ',
     &  ((w2f_ztop-w2f_zbot)*pi*w2f_detrad**2) / 
     &  ((w2f_zmax-w2f_zmin)*pi*w2f_rad**2)
      
      write(*,*) ' '
      write(*,*) 'Is this suggested volume OK?'
      write(*,*) '  0 = no, I want to change'
      write(*,*) '  1 = yes'
      write(*,*) ' -1 = quit program'
      read(*,*) ok

      if (ok.eq.-1) stop

      if (ok.eq.0) then
        write(*,*) 'Enter z_max:'
        read(*,*) w2f_zmax
        write(*,*) w2f_zmax

        write(*,*) 'Enter z_min:'
        read(*,*) w2f_zmin
        write(*,*) w2f_zmin

        write(*,*) 'Enter radius:'
        read(*,*) w2f_rad
        write(*,*) w2f_rad
        write(*,*) ' '
        write(*,*) '### New simulation volume ###'
        goto 5
      endif

c...Use events as they are or reuse events until a given number of events
c      write(*,*) 
c     &  'Do you want to simulate a certain number of events or not?'
c      write(*,*) '  1 = yes, reuse until a given number of events'
c      write(*,*) '      in this case completely off tracks will also'
c      write(*,*) '      thrown away for the Sun.'
c      write(*,*) '  2 = no, use the events in file (w or w/o reusing)'
c      read(*,*) w2f_nevhow
c      write(*,*) w2f_nevhow

c...Number of times to use each muon
      write(*,*) 'Number of times to use each event (default = 1):'
      read(*,*) w2f_nuse
      write(*,*) w2f_nuse

c...Only keep good events
      write(*,*) 'Only keep good events that have a chance to trigger?'
      write(*,*) '  (1=yes, 0=no)'
      read(*,*) w2f_onlygood
      write(*,*) w2f_onlygood

c...Random number seed
      write(*,*) 'Give the random number seed (positive integer)'
      write(*,*) '  (0 = a seed will be generated from the time)'
      read(*,*) ranseed
      write(*,*) ranseed

c...Initialize counters
      w2f_evid=0
      wfnevok=0
      ngoodmax=0
      
c...Open f2000 file for output
      call OpenDat()

c...Read in events from WF file
 10   read(luwf,5005,end=100) wfline
      if (wfline(1:5).eq.'EVENT') then
        read(wfline,5001) wfscr5,wfevid
      endif

      wfnmu=0
 20   read(luwf,5005,end=100) wfline
      if (wfline(1:2).eq.'EE') goto 50
      read(wfline,5002) wfpart,wfth,wfphi,wfe
      if (wfpart(1:2).eq.'mu') then 
        wfnmu=wfnmu+1
        wfmuth(wfnmu)=wfth
        wfmuphi(wfnmu)=wfphi
        wfmue(wfnmu)=wfe
      else
        write(*,*) 'WARNING in wf2f2000: Unknown particle found:',
     &    wfpart
        write(*,*) 'Please update code to include it.'
      endif
      goto 20
  
 50   continue ! Event read, write it to f2000

      ngood=0 ! counter for reuse of good events
      do j=1,w2f_nuse
        w2f_evid=w2f_evid+1


        if (wfwh.eq.'ea'.or.wfwh.eq.'EA') then ! ***** Earth *****

c...Spread in volume
          v_rad=sqrt(distuni(0.0d0,w2f_rad**2))
          v_z=distuni(w2f_zmin,w2f_zmax)
          v_phi=distuni(0.0d0,2.0d0*pi)

c...Now check if this track can have a chance to trigger
          if (w2f_onlygood.eq.1) then
c...Determine coordinates
            x0=v_rad*cos(v_phi)
            y0=v_rad*sin(v_phi)
            z0=v_z
            good=0
c...Go through the muons
            do i=1,wfnmu
              th=pi-(wfmuth(i)*pi/180.0)
              phi=wfmuphi(i)*pi/180.0
              nx=-sin(th)*cos(phi)
              ny=-sin(th)*sin(phi)
              nz=-cos(th)
              lmu=murang(wfmue(i))*1.1d0
              zmax=z0+lmu*nz
              if (zmax.ge.(w2f_zbot-100.0d0)) good=1
            enddo
            if (good.eq.0) then 
c              write(*,*) 'Skipped: z0=',z0,' lmu=',lmu,' zmax=',zmax
              goto 60 ! skip this event
            endif
          endif

         
c...The event is OK, let's put it to the f2000 file
          ngood=ngood+1
          wfnevok=wfnevok+1  ! One more OK event
          call frdmc_new_fmevt()
          do i=1,wfnmu

c...Put in a muon only
            call frdmc_new_gen(ntrack_e+1,result)

            th=pi-(wfmuth(i)*pi/180.0)
            phi=wfmuphi(i)*pi/180.0

            ntag_gen=i
            id_gen = 5
            e_gen=wfmue(i)
            t_gen=0.
          
c...Determine coordinates
            x_gen=v_rad*cos(v_phi)
            y_gen=v_rad*sin(v_phi)
            z_gen=v_z

            px_gen = e_gen*cos(phi)*sin(th)
            py_gen = e_gen*sin(phi)*sin(th)
            pz_gen = e_gen*cos(th)
            costh_gen=cos(th)
            phi_gen=phi
            length_gen=murang(wfmue(i))
            e_gen=e_gen*1000.0 ! convert to MeV
          enddo

        else ! ***** Sun *****

c...First choose a direction for the neutrino
          if (nu_way.eq.1) then     ! Fix theta
            nu_th=nu_th0
          else                      ! Spread theta
            nu_t=distuni(nu_t1,nu_t2)
            nu_th=90.0d0-w2f_suinc*cos(nu_t*2*pi)
          endif
          nu_phi=distuni(0.0d0,360.0d0)
c...Spread in volume
          v_rad=sqrt(distuni(0.0d0,w2f_rad**2))
          v_z=distuni(w2f_zmin,w2f_zmax)
          v_phi=distuni(0.0d0,2.0d0*pi)

c...Now check if this track can have a chance to trigger
          if (w2f_onlygood.eq.1) then
c...Determine coordinates
            x0=v_rad*cos(v_phi)
            y0=v_rad*sin(v_phi)
            z0=v_z
            good=0
c...Go through the muons
            do i=1,wfnmu
              numu_th=wfmuth(i)
              numu_phi=wfmuphi(i)
              call angadd(nu_th,nu_phi,numu_th,numu_phi,mu_th,mu_phi)
              th=mu_th*pi/180.0d0
              phi=mu_phi*pi/180.0d0
              nx=-sin(th)*cos(phi)
              ny=-sin(th)*sin(phi)
              nz=-cos(th)
              lmu=murang(wfmue(i))*1.1d0
c...Now determine how close to the z-axis it comes
              lmin=min(-(x0*nx+y0*ny)/(nx**2+ny**2),lmu)
              lmin=max(lmin,0.0d0)
              rmin=sqrt((x0+lmin*nx)**2+(y0+lmin*ny)**2) ! closest to z-axis
              if (rmin.lt.(w2f_detrad+100.0d0)) good=1
            enddo
            if (good.eq.0) then
c              write(*,*) 'Skipped: x0=',x0,' y0=',y0,' z0=',z0
c              write(*,*) '         nu_th=',nu_th,' nu_phi=',nu_phi
c              write(*,*) '         nx=',nx,' ny=',ny,' nz=',nz
c              write(*,*) '         lmu=',lmu,' lmin=',lmin,
c     &          ' rmin=',rmin
              goto 60 ! skip this event
            endif
          endif

c...The event is OK, let's put it to the f2000 file
          ngood=ngood+1
          wfnevok=wfnevok+1  ! One more OK event
          call frdmc_new_fmevt()

c...First put a neutrino in
          call frdmc_new_gen(ntrack_e+1,result)
          ntag_gen=1
          id_gen = 202
          t_gen=0.
          
c...Determine coordinates
          x_gen=v_rad*cos(v_phi)
          y_gen=v_rad*sin(v_phi)
          z_gen=v_z

          costh_gen=cos(nu_th*pi/180.0d0)    ! Fix 010402
          phi_gen=nu_phi*pi/180.0d0          ! Fix 010402

c...Then put the muon(s) in
          do i=1,wfnmu
            call frdmc_new_gen(ntrack_e+1,result)
            numu_th=wfmuth(i)
            numu_phi=wfmuphi(i)
            call angadd(nu_th,nu_phi,numu_th,numu_phi,mu_th,mu_phi)
            th=mu_th*pi/180.0d0
            phi=mu_phi*pi/180.0d0

            ntag_gen=i+1
            id_gen = 5
            nparent_gen = 1
            e_gen=wfmue(i)
            t_gen=0.
          
c...Spread in volume
            x_gen=v_rad*cos(v_phi)
            y_gen=v_rad*sin(v_phi)
            z_gen=v_z

            px_gen = e_gen*cos(phi)*sin(th)
            py_gen = e_gen*sin(phi)*sin(th)
            pz_gen = e_gen*cos(th)
            costh_gen=cos(th)
            phi_gen=phi
            length_gen=murang(wfmue(i))
            e_gen=e_gen*1000.0 ! convert to MeV
          enddo
        endif

        nrun_e=wfrunid
        enr_e=w2f_evid
      
        call frdmc_wevt(result)

 60     continue
      enddo
      ngoodmax=max(ngood,ngoodmax)

      goto 10

 100  continue ! End of WF file found
      close(luwf)

      call frdmc_mcclose(RDMC_WRITE_MODE,result)
      if (result.ne.0) then
        write(*,*) 'ERROR in closing datafile, result:',result
      endif
      write(*,*) 'Number of vertices spread: ',wfnmuon*w2f_nuse
      write(*,*) 'Number of vertices written to file: ',wfnevok
      if (w2f_onlygood.eq.1) then
        write(*,*) 'Max number of kept events per event: ',
     &    ngoodmax
      endif
      END


***********************************************************************
*** STARTUP - print velcome message
***********************************************************************

      subroutine startup
      implicit none
      include 'wf.inc'
      include 'w2f.inc'
      integer i

      write(*,*) ('*',i=1,50)
      write(*,*) '          Welcome to wf2f2000 ',w2fver
      write(*,*) '            ',w2fdate
      write(*,*) ' '
      write(*,*) '               Joakim Edsjo'
      write(*,*) '              edsjo@physto.se'
      write(*,*) ('*',i=1,50)

      end


***********************************************************************
*** CHECKGEOM - print the geometry used and give the option to
*** change it.
***********************************************************************

      subroutine checkgeom
      implicit none
      include 'wf.inc'
      include 'w2f.inc'
      integer ok

      write(*,*) ' '
      write(*,*) '#########################'
      write(*,*) '### DETECTOR GEOMETRY ###'
      write(*,*) '#########################'
      write(*,*) ' '
      write(*,*) 'Current detector geometry (in meters):'
      write(*,*) '  z coordinate, top of the detector:    ',
     &  w2f_ztop
      write(*,*) '  z coordinate, bottom of the detector: ',
     &  w2f_zbot
      write(*,*) '  radius of the detector:               ',
     &  w2f_detrad


 10   write(*,*) ' '
      write(*,*) 'Is this OK? (0=no, 1=yes)'
      read(*,*) ok

      if (ok.eq.1) then
        write(*,*) 'Detector geometry saved.'
        return
      else  ! change detector geomtry
        write(*,*) 'New z coordinate for top of the detector:'
        read(*,*) w2f_ztop

        write(*,*) 'New z coordinate for bottom of the detector:'
        read(*,*) w2f_zbot

        write(*,*) 'New radius of the detector:'
        read(*,*) w2f_detrad

        write(*,*) ' '
        write(*,*) 'New detector geometry (in meters):'
        write(*,*) '  z coordinate, top of the detector:    ',
     &    w2f_ztop
        write(*,*) '  z coordinate, bottom of the detector: ',
     &    w2f_zbot
        write(*,*) '  radius of the detector:               ',
     &    w2f_detrad
      endif

      goto 10

      end



***********************************************************************
*** Open f2000 file                                                 ***
***********************************************************************

      SUBROUTINE OpenDat()
      IMPLICIT none
      include 'rdmc.inc'
      include 'wf.inc'
      include 'w2f.inc'

c#include "chwvar.inc"
c#include "gencom.inc"
c#include "grinfo.inc"
c#include "grcure.inc"
      CHARACTER*255 datafile
      CHARACTER*250 coment
      CHARACTER*250 comnt(20)
      INTEGER result
      INTEGER i,j,igmax,jy,jm,jd
      INTEGER ijunk
      CHARACTER*7 version
      character*9 month(12)
      data month/'January','February','March','April','May','June',
     &  'July','August','September','October','November','December'/

C- Temporary settings to match Basiev file
      ijunk   = 0    
C-
 20     WRITE (*,*) 
     $  'Give the name of the Outfile-name'//
     $ '(.gz for automatic gzip on unix)'
        READ(*,'(A)',ERR=20) datafile

       	CALL frdmc_init_dummy()
	CALL frdmc_mcopen(datafile//CHAR(0),RDMC_WRITE_MODE
     $       ,F2000_F,result)
	IF (result.ne.0) THEN
          write(*,*) 
          goto 99
        ENDIF
C
C get some other stuff
 30     WRITE(*,*) 'Please give the run number (int)'
        READ(*,*,ERR=30) wfrunid


C history - WimpSimp
        WRITE(coment,'(A,A7,A,2(A,I7),2A)') 
     $       wfprog

        CALL frdmc_add_history(coment//CHAR(0),result)

C write some comments (LEUs style) Modified by AG and JE

        write(comnt(1),*) '!  WimpSimp run parameters:'
        write(comnt(2),*) '!  ',wfpara
        write(comnt(3),*) '!  ',wfdate
        write(comnt(4),*) '!  '
        do i=1,4
           call frdmc_add_hdcomment(comnt(i)(2:80)//char(0),result)
           if (result .ne. 0) then
              write(*,*) 'OpenDat: problem while writing ',
     &             'comment line',i,' result ',result
              goto 99
           endif
        enddo

C history - wf2f2000
        WRITE(coment,'(A,A,A,A,A,A,A)')
     $    'wf2f2000 (v ',w2fver,') ',wffile,
     $    ' (version from ',w2fdate,')'
        CALL frdmc_add_history(coment//CHAR(0),result)

C write some comments (LEUs style) Modified by AG and JE
        call idate(jm,jd,jy)
        if (jy.le.95) then 
          jy=2000+jy
        else
          jy=jy+1900
        endif

        write(comnt(1),*) '!  wf2f2000 run parameters:'
        write(comnt(2),121) '!  zmax=',w2f_zmax,'  zmin=',w2f_zmin,
     &    '  radius=',w2f_rad
        write(comnt(3),123) '!  nuse=',w2f_nuse,
     &    '  nev_tot=',wfnmuon*w2f_nuse
        write(comnt(4),122) '!  Date: ',month(jm),jd,jy
        write(comnt(5),*) '!  '
 121    format(A,F8.1,A,F8.1,A,F8.1,A,I6)        
 122    format(A,A,I02,', ',I4)
 123    format(A,I6,A,I10)        
        do i=1,5
           call frdmc_add_hdcomment(comnt(i)(2:80)//char(0),result)
           if (result .ne. 0) then
              write(*,*) 'OpenDat: problem while writing ',
     &             'comment line',i,' result ',result
              goto 99
           endif
        enddo

c now write the header
         CALL frdmc_warr(result)
         IF (result.ne.0) THEN
          WRITE(*,'(A)')' Going to crash...' 
          GOTO 98
         ENDIF
         write(*,*) ' Leaving OpenDat peacefully...'
         RETURN
c----------------------------------------------------------------------
c
 98     WRITE(*,*) ' ERROR writing datfile header, result:',result
        STOP

 99     WRITE(*,*) ' ERROR opening datfile:',datafile
     $       ,', result:',result
        STOP     
C_________________________________________________________________


C        RETURN
        END


******************************************************************************
*** FUNCTION MURANG calculates the range in the ice for a muon of
*** a given energy (in GeV). Unit: m
******************************************************************************

      REAL*8 FUNCTION MURANG(EMU)
      IMPLICIT NONE
      
C------------------------ Variables ------------------------------------

      REAL*8 EMU, ALPHAM, BETAM, MMU
      real*8 alpha,beta,dens

      PARAMETER (MMU=0.105658d0)                   ! GeV
      parameter(alpha=0.00260d0,beta=3.49d-6)
      parameter(dens=0.92d0)

C-----------------------------------------------------------------------

C...Calculate range of muons by assuming dE/dX=-alpha-beta*Emu
      ALPHAM=ALPHA*DENS   ! GeV/cm
      BETAM=BETA*DENS     ! cm**-1
      MURANG = 1.0/BETAM*LOG((SQRT(EMU**2-MMU**2)+ALPHAM/BETAM)/
     &  (ALPHAM/BETAM))                   ! cm
c...Convert to meters
      murang=murang*0.01

      END


***********************************************************************
*** SIMVOL - find the optimal generation volume                     ***
***********************************************************************

      subroutine simvol(a,b)
* Goes through the WF-file given in wffile and finds the values
* of a and b that are sufficient for the generation volume.
* a is the extension outside the detector radius and b is the
* extension below the detector
* NOTE: The routine checks if the file is for Sun simulation or
*       Earth simulation and calculates the values of a and b
*       thereafter

      implicit none
      include 'wf.inc'
      include 'w2f.inc'

      real*8 a,b,murang,aev,bev,rang

      write(*,*)      
      write(*,*) '#########################'
      write(*,*) '### SIMULATION VOLUME ###'
      write(*,*) '#########################'
      write(*,*) ' '
      write(*,*) 
     &  'Scanning input file to find optimal simulation volume...'
      a=0.0d0
      b=0.0d0
      
c...Open WF file for input
      open(unit=luwf,file=wffile,status='old',form='formatted')
      read(luwf,'(1x,A)') wfver
      read(luwf,'(1x,A)') wfprog
      read(luwf,'(1x,A)') wfpara
      read(luwf,'(1x,A)') wfdate

      if (wfver.ne.'WF 1.0') then
        write(*,*)
     &        'WARNING: A different version of the WF format found.'
        write(*,*) 'Please check the code for consistency.'
      endif

      read(wfpara,5010) wfmneu,wfwh,wfch,wfnmuon

 10   read(luwf,5005,end=100) wfline
      if (wfline(1:2).eq.'mu') then
        read(wfline,5002) wfpart,wfth,wfphi,wfe
        rang=murang(wfe)
        if (wfwh.eq.'ea'.or.wfwh.eq.'EA') then   ! Earth
          aev=rang*sin(wfth*pi/180.0d0)
          bev=rang*cos(wfth*pi/180.0d0)
        else                                     ! Sun
          aev=rang
          bev=rang*sin((abs(wfth)+abs(w2f_suinc))*pi/180.0d0)
        endif

        if (aev.gt.a) a=aev
        if (bev.gt.b) b=bev
      endif
      goto 10
      
 100  close(luwf)
      write(*,*) 'Done.'

      return
      end
      

***********************************************************************
*** 


***********************************************************************
*** DISTUNI - distributes random numbers uniformly between a and b  ***
***********************************************************************      
      
      real*8 function distuni(a,b)
* Distribute uniformly between a and b
      implicit none
      
      real*8 a,b
      integer iseed
      real dummy,rnunif
      logical first
      data first /.true./
      save first
      
      if (first) then
        call start_ran(iseed)
        first=.false.
      endif
      
      distuni=a+(b-a)*dble(rnunif(dummy))
      return
      end

      
**********************************************************************
*** Random number generator from Ch. Walck                         ***
**********************************************************************
      
      subroutine start_ran(iseed)
C INITILAIZE RANDOM NUMBER GENERATION
*
      integer ranseed
      common /w2f_ran/ranseed
      save /w2f_ran/

      if (ranseed.eq.0) then  ! generate seed from time
        call rnclck ( iseed )
      else
        iseed=ranseed
      endif

      call rninit ( iseed )
      end 


      SUBROUTINE RNCLCK ( ISEED )
*
*            Clock start of random number generator
*
*     Calculate number of seconds from 1 January 1989 modulus 921350144
*     which may be used as start seed for the sequence. This implies
*     that one avoids using the same start sequence in any simulation
*     unless two jobs are ran in parallel during the same second!
*     The code previously used the CERN library routine DATIMH to
*     obtain the system date and time. This, however, is a unnecessary
*     detour in achieving this. A new subroutine RNDATE has been
*     supplied which directly uses system functions for date and time
*     on most computers.
*
*     Working in several laboratories one may, as has been done for
*     DELPHI, construct a value from a lab-code and the number of
*     seconds past during the current year.
*
*                                                    Ch.Walck 1989-12-21
*                                                     revised 1990-09-22
*                                                     bug fix 1994-09-05
*
      SAVE IMD
      DIMENSION IMD(12)
      DATA IMD / 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334 /
*
*     Get system date and time
*
      CALL RNDATE ( JY, JM, JD, IH, IM, IS )
*
*     Calculate number of days in previous years from 1 January 1989
*
      IY = 1900 + JY
      IF ( JY .LT. 89 ) IY = 2000 + JY
      IDY = 0
      IF ( IY .GT. 1989 ) THEN
         DO 100 I = 1989, IY-1
            JDY = 365
*           Code valid until 2100 which is not a leap-year (2000 is! 940905)
            IF ( MOD(I,4).EQ.0 ) JDY = 366
            IDY = IDY + JDY
  100    CONTINUE
      END IF
*
*     Number of days from 1 January until 1st current month
*     (treat leap-years as well)
*
      LY =0
*     Code valid until 2100 which is not a leap-year (2000 is! 940905)
      IF ( MOD(JY,4).EQ.0 ) LY = 1
      ISHIFT = IMD(JM)
      IF ( JM .GT. 2 ) ISHIFT = IMD(JM) + LY
*
*     Seconds since January 1st 1989 at 00:00:00 (=0)
*
      ISEED = IS + 60 * ( IM + 60 * ( IH + 24 * ( JD-1+ISHIFT+IDY ) ) )
*
*     Bring packed seed into range (this modulus will zero the ISEED at
*     18:35:44 on March 13 2018 and at 13:11:28 on May 24 2047, ... )
*
      ISEED = MOD(ISEED,921350144)
*
      WRITE (6,1003) JD, JM, JY, IH, IM, IS, ISEED
 1003 FORMAT (/' RNCLCK: date and time is',I3,'-',I2.2,'-',I2.2,I3.2,':'
     +       ,I2.2,':',I2.2,', clock start at seed',I10/)
      RETURN
      END

      SUBROUTINE RNDATE ( JY, JM, JD, IH, IM, IS )
*
*     Routine to give current date and time. This routine replaces the
*     old DATIMH routine from the CERN library which was an unnecessary
*     detour in achieving this information. The routine is not fully
*     tested on all computers but hopefully works anyway. If any problem
*     is encountered the self-tag DATIMH activates the old code.
*
*          JY - Year (modulus 100)        IH - Hours
*          JM - Month                     IM - minutes
*          JD - Date                      IS - seconds
*
*                                                    Ch.Walck 1990-09-22
*
*     System calls on VAX
*
      CHARACTER*8 HMS
      CALL IDATE ( JM, JD, JY )
      CALL TIME  ( HMS )
      READ (HMS,1001) IH, IM, IS
 1001 FORMAT (I2,1X,I2,1X,I2)
      RETURN
      END

      SUBROUTINE RNINIT ( ISEED )
*
*     Initialize random number generator by Marsaglia & Zaman (& Tsang).
*
*     The initial values for U(97) are generated by using a 3-lag
*     Fibonacci generator with 3 initial seeds and a congruential
*     generator with one initial seed.
*
*     - The values of the three initial seeds for the 3-lag Fibonacci
*       generator may be in the range 1 to 178. This generator has a
*       period of 32044 in about half of all cases. In about a quarter
*       of all cases each the period is 16022 and 8011. In a few cases
*       with combinations like 1, 1, 178 or 178, 1, 178 the period is
*       only 1, 2 or 4. The authors proposes to avoid the combination
*       1, 1, 1 but we choose to be somewhat more restrictive and use
*       only values from 2 to 177 for the three seeds.
*     - The initial seed for the congruential generator may be in the
*       range 0 to 168. The period of this generator is 169 regardless
*       of the initial seed value.
*
*     The four initial seeds are packed into one integer which can be
*     regarded as a four digit number with a variable base for each
*     digit. This in order to get a one-to-one correspondence between
*     the four seeds and the big integer without leaving any holes.
*     With I, J, K being the seeds of the Fibonacci generator and L the
*     seed of the congruential generator we pack them as:
*        ISEED = (I-2)*176*176*169 + (J-2)*176*169 + (K-2)*169 + L
*     ISEED can thus have the range 0 to 169*176**3-1=921350143 all
*     values giving a legal and unique I, J, K, L combination.
*
*                                                    Ch.Walck 1989-12-21
*
      COMMON /RNCMMN/ LAG1, LAG2, U(97), C, CD, CM
*
      LAG1 = 97
      LAG2 = 33
*
*     Abort run if ISEED value is out of its valid range
*
      IF ( ISEED.LT.0 .OR. ISEED.GT.921350143 ) THEN
         WRITE (6,1001) ISEED
 1001    FORMAT (//' RNINIT: error - ISEED out of range, ISEED=',I11
     +          ,'), run aborted.'//)
         STOP
      END IF
*
*     Unpack seeds for 3-lag Fibonacci and congruential generator
*
      I = MOD(ISEED/169/176/176,176) + 2
      J = MOD(ISEED/169/176,176) + 2
      K = MOD(ISEED/169,176) + 2
      L = MOD(ISEED,169)
*
*     Floating point arithmetics version
*
      DO 200 II = 1, 97
         S = 0.0
         T = 0.5
         DO 100 JJ = 1, 24
            M = MOD(MOD(I*J,179)*K,179)
            I = J
            J = K
            K = M
            L = MOD(53*L+1,169)
            IF ( MOD(L*M,64) .GE. 32 ) S = S + T
            T = 0.5 * T
  100    CONTINUE
         U(II) = S
  200 CONTINUE
      C  =   362436.0 / 16777216.0
      CD =  7654321.0 / 16777216.0
      CM = 16777213.0 / 16777216.0
      RETURN
      END

      FUNCTION RNUNIF ( DUM )
*
*     Universal pseudorandom number generator by George Marsaglia & Arif
*     Zaman as described in the report FSU-SCRI-87-50, Florida State
*     University, 1987 (later on published together with a third author,
*     Wai Wan Tsang, in Stat. & Prob. Lett. 9 (1990) 35-39).
*
*     Initialize by a call to RNINIT and make use of, if wanted, the
*     utilities to pack/unpack seeds or use a clock-start.
*
*                                                    Ch.Walck 1989-12-21
*
      COMMON /RNCMMN/ LAG1, LAG2, U(97), C, CD, CM
*
*     Floating point arithmetics version
*
    1 UNI = U(LAG1) - U(LAG2)
      IF ( UNI .LT. 0.0 ) UNI = UNI + 1.0
      U(LAG1) = UNI
      LAG1 = LAG1 - 1
      IF ( LAG1 .EQ. 0 ) LAG1 = 97
      LAG2 = LAG2 - 1
      IF ( LAG2 .EQ. 0 ) LAG2 = 97
      C = C - CD
      IF ( C .LT. 0.0 ) C = C + CM
      UNI = UNI - C
      IF ( UNI .LT. 0.0 ) UNI = UNI + 1.0
*     Avoid returning zero
      IF ( UNI .EQ. 0.0 ) GO TO 1
      RNUNIF = UNI
      RETURN
      END

c      SUBROUTINE TIMEX ( T )
c      INCLUDE '($JPIDEF)'
c      CHARACTER*20 STRING
c      ISTAT = LIB$GETJPI ( JPI$_CPUTIM, , , ITIME, STRING, LEN )
c      IF ( .NOT.ISTAT ) CALL LIB$SIGNAL (%VAL(ISTAT))
c      T = FLOAT(ITIME) / 100.
c      RETURN
c      END

      FUNCTION RNNRMT ( XMU, XSIG )
C-----------------------------------------------------------------------
C
C...       FUNCTION RNNRMT ( XMU, XSIG )                    Ch.Walck    861005
C.
C.    Gaussian random number with mean XMU and standard deviation XSIG.
C.
C.    F(X;XMU,XSIG) = EXP(-((X-XMU)/XSIG)**2/2) / SQRT(2*PI) / XSIG
C.
C.    Variable:          X        (real)
C.    Parameters:        XMU      (real)
C.                       XSIG > 0 (real)
C.
C.    Expectation value: E(Z) = XMU
C.    Variance:          V(Z) = XSIG**2
C.    Skewness:         G1(Z) = 0
C.    Kurtosis:         G2(Z) = 0
C.
C.    Technique:
C.    Trapezoidal method of Ahrens (1972) as given in the article
C.    "Computer Methods for Sampling From the Exponential and Normal
C.    Distributions" by J.H.Ahrens and U.Dieter in Communications of
C.    the ACM Vol.15 (1972) 873. In fact this is the fastest of the
C.    routines given here! (However, the CERN routine NORRAN, coded
C.    in assembler language, is faster.)
C.
C.----------------------------------------------------------------------
      PARAMETER (   A=0.919544405706926,    A1=0.954057274493068,
     +             A2=0.978195649518392,    A3=0.993692071999222,
     +             X1=0.2897295736,         X2=1.84039874739771,
     +             X3=2.11402808333742,   X3SQ=4.46911473713927,
     +           DX12=1.55066917379771,   DX23=0.273629335939706,
     +           XINT=0.443299125820220, SLOPE=0.209694057195486,
     +            YMX=0.398942280401433,    YT=0.382544556042518,
     +             Y1=0.016397724358915,    Y2=0.015974522655238,
     +             Y3=0.042702581590795,    Q1=2.40375765693742,
     +             Q2=1.983915619969365)
C
      U  = RNUNIF(DUM)
      U0 = RNUNIF(DUM)
C
C     From maximal trapezoid inscribed in N(0,1)
C    -2.11 < X < 2.11 with P = 91.95 %
C
      IF ( U .LT. A ) THEN
         X = Q1*U0 + Q2*U - X3
         GO TO 100
C
C     From tail of N(0,1)
C     X > 2.11 with P = 3.45 % (sign decided below)
C
      ELSE IF ( U .LT. A1 ) THEN
    1    X  = X3SQ-2.0*LOG(RNUNIF(DUM))
         IF ( X*RNUNIF(DUM)**2 .GT. X3SQ ) GO TO 1
         X = SQRT(X)
C
C     From 2nd region between N(0,1) and trapezoid
C     0.29 < X < 1.84 with P = 2.41 % (sign decided below)
C
      ELSE IF ( U .LT. A2 ) THEN
    2    X = X1 + DX12 * RNUNIF(DUM)
         IF ( YMX*EXP(-X**2/2.0)-XINT+X*SLOPE.LT.Y2*RNUNIF(DUM)) GO TO 2
C
C     From 3rd region between N(0,1) and trapezoid
C     1.84 < X < 2.11 with P = 1.55 % (sign decided below)
C
      ELSE IF ( U .LT. A3 ) THEN
    3    X = X2 + DX23 * RNUNIF(DUM)
         IF ( YMX*EXP(-X**2/2.0)-XINT+X*SLOPE.LT.Y3*RNUNIF(DUM)) GO TO 3
C
C     From 1st (top) region between N(0,1) and trapezoid
C     0 < X < 0.29 with P = 0.63 % (sign decided below)
C
      ELSE
    4    X = X1 * RNUNIF(DUM)
         IF ( YMX*EXP(-X**2/2.0)-YT .LT. Y1*RNUNIF(DUM) ) GO TO 4
      END IF
C
C     Attach sign
C
      IF ( U0 .GE. 0.5 ) X = -X
C
  100 RNNRMT = X*XSIG + XMU
      RETURN
      END


***********************************************************************
*** SUBROUTINE ANGADD takes a direction theta_1, phi_1 and rotates  ***
*** it an angle theta_2 in direction phi_2 (phi_2=0 is towards      ***
*** the z-axis and phi_2=90 degrees is towards phi-hat).            ***
*** The resulting angles theta_3 and phi_3 are calculated.          ***
*** Units: degrees                                                  ***
***********************************************************************

      subroutine angadd(th1,phi1,th2,phi2,th3,phi3)
      implicit none

      real*8 th1,phi1,th2,phi2,th3,phi3
      real*8 cth1,cth2,sth1,sth2,cphi2,cth3,sth3,cdphi,dphi
      real*8 pi
      parameter (pi=3.141592653589793238d0)

      cth1=cos(th1*pi/180.0d0)
      cth2=cos(th2*pi/180.0d0)
      sth1=sin(th1*pi/180.0d0)
      sth2=sin(th2*pi/180.0d0)
      cphi2=cos(phi2*pi/180.0d0)

      cth3=cth1*cth2+sth1*sth2*cphi2
      cth3=min(cth3,1.0d0)
      cth3=max(cth3,-1.0d0)

      th3=acos(cth3)*180.0d0/pi

      sth3=sin(th3*pi/180.0d0)

      cdphi=(cth2-cth1*cth3)/(sth1*sth3)
      cdphi=min(cdphi,1.0d0)
      cdphi=max(cdphi,-1.0d0)

      dphi=acos(cdphi)*180.0d0/pi
      if (phi2.le.180.0d0) then
        phi3=phi1+dphi
      else
        phi3=phi1-dphi
      endif

      if (phi3.lt.0.0d0)   phi3=phi3+360.0d0
      if (phi3.gt.360.0d0) phi3=phi3-360.0d0

      return
      end
